/*
  This file is part of Anastasis
  Copyright (C) 2019-2021, 2026 Anastasis SARL

  Anastasis is free software; you can redistribute it and/or modify it under the
  terms of the GNU Affero General Public License as published by the Free Software
  Foundation; either version 3, or (at your option) any later version.

  Anastasis is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License along with
  Anastasis; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
*/
/**
 * @file stasis/anastasis-dbinit.c
 * @brief Create tables for the merchant database.
 * @author Dennis Neufeld
 * @author Dominik Meister
 */
#include "platform.h"
#include "anastasis_util_lib.h"
#include "anastasis_database_lib.h"


/**
 * Return value from main().
 */
static int global_ret;

/**
 * -r option: do full DB reset
 */
static int reset_db;

/**
 * -g option: do garbate collection
 */
static int gc_db;

/**
 * Main function that will be run.
 *
 * @param cls closure
 * @param args remaining command-line arguments
 * @param cfgfile name of the configuration file used (for saving, can be NULL!)
 * @param cfg configuration
 */
static void
run (void *cls,
     char *const *args,
     const char *cfgfile,
     const struct GNUNET_CONFIGURATION_Handle *cfg)
{
  if (GNUNET_OK !=
      ANASTASIS_DB_init_admin (cfg))
  {
    fprintf (stderr,
             "Failed to initialize database.\n");
    global_ret = EXIT_NOTINSTALLED;
    return;
  }
  if (reset_db)
  {
    if (GNUNET_OK !=
        ANASTASIS_DB_drop_tables ())
    {
      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                  "Could not drop tables as requested. Either database was not yet initialized, or permission denied. Consult the logs. Will still try to create new tables.\n");
    }
  }
  if (GNUNET_OK !=
      ANASTASIS_DB_create_tables ())
  {
    global_ret = EXIT_FAILURE;
    ANASTASIS_DB_fini ();
    return;
  }
  /* Finish what the stasis-0004 migration could not: SQL has no access to
     [anastasis] CURRENCY, so the IBAN transfers it widened were left with a
     NULL currency.  Idempotent, and a no-op for a freshly created database. */
  {
    enum GNUNET_DB_QueryStatus qs;

    qs = ANASTASIS_DB_update_auth_iban_in_currency ();
    if (0 > qs)
    {
      fprintf (stderr,
               "Failed to set the currency of migrated IBAN transfers.\n");
      global_ret = EXIT_FAILURE;
      ANASTASIS_DB_fini ();
      return;
    }
    if (0 < qs)
      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                  "Stamped currency onto %d migrated IBAN transfer(s)\n",
                  (int) qs);
  }
  if (gc_db)
  {
    struct GNUNET_TIME_Absolute expire_backups;
    struct GNUNET_TIME_Absolute expire_payments;
    struct GNUNET_TIME_Absolute now;
    struct GNUNET_TIME_Relative backup_grace;
    struct GNUNET_TIME_Relative payment_retention;

    /* How long expired data is kept before it is collected is a
       data-minimization decision, so let the operator make it. */
    if (GNUNET_OK !=
        GNUNET_CONFIGURATION_get_value_time (cfg,
                                             "anastasis",
                                             "GC_BACKUP_GRACE",
                                             &backup_grace))
      backup_grace = GNUNET_TIME_relative_multiply (
        GNUNET_TIME_UNIT_MONTHS,
        6);
    if (GNUNET_OK !=
        GNUNET_CONFIGURATION_get_value_time (cfg,
                                             "anastasis",
                                             "GC_PAYMENT_RETENTION",
                                             &payment_retention))
      payment_retention = GNUNET_TIME_relative_multiply (
        GNUNET_TIME_UNIT_YEARS,
        10);
    now = GNUNET_TIME_absolute_get ();
    expire_backups = GNUNET_TIME_absolute_subtract (now,
                                                    backup_grace);
    expire_payments = GNUNET_TIME_absolute_subtract (now,
                                                     payment_retention);
    if (0 > ANASTASIS_DB_do_gc (
          expire_backups,
          expire_payments))
    {
      fprintf (stderr,
               "Garbage collection failed!\n");
      /* A cron job or timer unit driving `-g' must be able to see that
         nothing was reclaimed. */
      global_ret = EXIT_FAILURE;
    }
  }
  ANASTASIS_DB_fini ();
}


/**
 * The main function of the database initialization tool.
 * Used to initialize the Anastasis' database.
 *
 * @param argc number of arguments from the command line
 * @param argv command line arguments
 * @return 0 ok, 1 on error
 */
int
main (int argc,
      char *const *argv)
{
  struct GNUNET_GETOPT_CommandLineOption options[] = {
    GNUNET_GETOPT_option_flag ('g',
                               "garbagecollect",
                               "remove state data from database",
                               &gc_db),
    GNUNET_GETOPT_option_flag ('r',
                               "reset",
                               "reset database (DANGEROUS: all existing data is lost!)",
                               &reset_db),

    GNUNET_GETOPT_OPTION_END
  };
  enum GNUNET_GenericReturnValue ret;

  ret = GNUNET_PROGRAM_run (ANASTASIS_project_data (),
                            argc, argv,
                            "anastasis-dbinit",
                            "Initialize anastasis database",
                            options,
                            &run, NULL);
  if (GNUNET_SYSERR == ret)
    return EXIT_INVALIDARGUMENT;
  if (GNUNET_NO == ret)
    return EXIT_SUCCESS;
  return global_ret;
}


/* end of anastasis-dbinit.c */
